@@ -11,16 +11,16 @@
     IBOutlet NSMenuItem *baseMenu;
     IBOutlet NSMenuItem *saveMenuItem;
     IBOutlet NSMenuItem *saveAsMenuItem;
+    IBOutlet NSMenuItem *sizeToggleMenu;
 
     IBOutlet NSWindow *mainWindow;
     IBOutlet NSWindow *inspectorWindow;
-	IBOutlet NSWindow *persVarsWindow;
+    IBOutlet NSWindow *persVarsWindow;
     IBOutlet InspectorController *inspector;
 
     IBOutlet NSButton *capsLockKey;
     IBOutlet NSButton *shiftKey1;
     IBOutlet NSButton *shiftKey2;
-    IBOutlet NSButton *engineeringNotation;
     IBOutlet NSButton *historyDuplicates;
     IBOutlet NSButton *pickyVariables;
     IBOutlet NSButton *printPrefixes;
@@ -34,30 +34,36 @@
     IBOutlet NSButton *precisionGuard;
     IBOutlet NSButton *printInts;
     IBOutlet NSButton *simpleCalculator;
-	IBOutlet NSButton *printDelimiters;
+    IBOutlet NSButton *printDelimiters;
+    IBOutlet NSButton *livePrecision;
+    IBOutlet NSButton *cModPref;
 
     IBOutlet NSTextField *AnswerField;
     IBOutlet NSTextField *ExpressionField;
     IBOutlet NSTextField *limitHistoryLen;
     IBOutlet NSTextField *limitHistoryLenTag;
-	IBOutlet NSTextField *bitsPref;
+    IBOutlet NSTextField *bitsPref;
+    IBOutlet NSTextField *sliderPref;
 
     IBOutlet NSDrawer *baseDrawer;
 	
-	IBOutlet NSStepper *bitsStepper;
+    IBOutlet NSStepper *bitsStepper;
+    IBOutlet NSStepper *sliderStepper;
 
     IBOutlet NSMatrix *outputFormat;
     IBOutlet NSMatrix *outputFormat2;
 
+    IBOutlet NSSlider *PrecisionSlider;
+    
     IBOutlet id errorController;
     IBOutlet id historyList;
-    IBOutlet id PrecisionSlider;
     IBOutlet id theKeyboard;
     IBOutlet id keypad;
 
     IBOutlet NSPanel *thePrefPanel;
     IBOutlet NSPanel *conversionWindow;
 
+    IBOutlet NSPopUpButton *engineeringNotation;
     IBOutlet NSPopUpButton *roundingIndication;
     IBOutlet NSPopUpButton *convertType;
 
@@ -65,7 +71,7 @@
     IBOutlet NSTableView *convertFrom;
     IBOutlet NSTableView *convertTo;
 	
-	IBOutlet NSApplication *meta;
+    IBOutlet NSApplication *meta;
 
     bool just_answered;
     NSView *superview;
@@ -88,9 +94,13 @@
 - (IBAction)save:(id)sender;
 - (IBAction)saveAs:(id)sender;
 - (IBAction)closeWindow:(id)sender;
+- (IBAction)quit:(id)sender;
+- (IBAction)savePrefs:(id)sender;
 
 - (void)displayAnswer;
 - (void)displayErrno:(int)err forFile:(NSString*)filename;
 - (BOOL)validateMenuItem:(NSMenuItem *)anItem;
+- (void)openBDrawer: (id) sender;
+- (void)openIDrawer: (id) sender;
 @end
 #endif
