@@ -4,6 +4,7 @@
 #define HAVE_MPFR_22
 #endif
 #include <stdlib.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <math.h>		       /* for HUGE_VAL */
 #include <float.h>		       /* for DBL_EPSILON */
@@ -30,8 +31,7 @@ char *strchr(), *strrchr();
 # endif
 #endif
 
-#include <gmp.h>
-#include <mpfr.h>
+#include "number.h"
 
 #include "uint32_max.h"
 #include "calculator.h"
@@ -42,11 +42,13 @@ char *strchr(), *strrchr();
 #include "add_commas.h"
 #include "list.h"
 #include "extract_vars.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
 /* variables everyone needs to get to */
-mpfr_t last_answer;
+Number last_answer;
 char *pretty_answer = NULL;
-gmp_randstate_t randstate;
 
 /* communication with the parser */
 char compute = 1;
@@ -55,6 +57,7 @@ unsigned int sig_figs = UINT32_MAX;
 /* communication with the frontend */
 char standard_output = 1;
 char not_all_displayed = 0;
+char *pa = NULL;
 
 struct _conf conf;
 
@@ -74,7 +77,7 @@ static int find_recursion(char *);
 static int find_recursion_core(List);
 static char *flatten(char *str);
 
-void parseme(char *pthis)
+void parseme(const char *pthis)
 {				       /*{{{ */
     extern int synerrors;
     short numbers = 0;
@@ -85,6 +88,7 @@ void parseme(char *pthis)
     compute = 1;
     sig_figs = UINT32_MAX;
 
+    Dprintf("parsing: %s\n", pthis);
     sanitized = (char *)strdup(pthis);
 
     /* Convert to standard notation (american comma and period) if there are
@@ -104,17 +108,13 @@ void parseme(char *pthis)
 	unsigned int i;
 
 	for (i = 0; i < strlen(sanitized); ++i) {
-	    char errmsg[1000];
-
 	    if ((conf.thou_delimiter != '.' && conf.dec_delimiter != '.' &&
 		 sanitized[i] == '.') || (conf.thou_delimiter != ',' &&
 					  conf.dec_delimiter != ',' &&
 					  sanitized[i] == ',')) {
 		// throw an error
-		snprintf(errmsg, 1000,
-			 "Improperly formatted numbers! (%c,%c)\n",
-			 conf.thou_delimiter, conf.dec_delimiter);
-		report_error(errmsg);
+		report_error("Improperly formatted numbers! (%c,%c)\n",
+			     conf.thou_delimiter, conf.dec_delimiter);
 		synerrors = 1;
 		break;
 	    } else if (sanitized[i] == conf.thou_delimiter)
@@ -147,10 +147,12 @@ void parseme(char *pthis)
 	free(sanitized);
 	sanitized = temp;
     }
+    /* reset the position tracker */
+    {
+	extern int column;
 
-
-    /* Hold my Place */
-//  stackcur = stacklast + 1;
+	column = 0;
+    }
     /* Evaluate the Expression
      * These two lines borrowed from:
      * http://www.bgw.org/tutorials/programming/c/lex_yacc/main.c
@@ -171,8 +173,8 @@ void parseme(char *pthis)
 	Dprintf("open_file\n");
 	retval = loadState(filename, 1);
 	if (retval) {
-	    report_error("Could not load file.");
-	    report_error((char *)strerror(retval));
+	    report_error("Could not load file (%s).",
+			 (char *)strerror(retval));
 	}
     }
   exiting:
@@ -181,6 +183,9 @@ void parseme(char *pthis)
     return;
 }				       /*}}} */
 
+/* this function should probably stop flattening if it sees a comment, but
+ * that's so rare (and hardly processor intensive) that it's not worth digging
+ * at the moment */
 static char *flatten(char *str)
 {				       /*{{{ */
     char *curs = str, *eov, *nstr;
@@ -246,23 +251,24 @@ static char *flatten(char *str)
 	// if it's a variable, evaluate it
 	a = getvar_full(varname);
 	if (!a.err) {		       // it is a var
-	    mpfr_t f;
+	    Number f;
 
-	    mpfr_init(f);
+	    num_init(f);
 	    if (a.exp) {	       // it is an expression
 		parseme(a.exp);
-		mpfr_set(f, last_answer, GMP_RNDN);
+		num_set(f, last_answer);
 	    } else {		       // it is a value
-		mpfr_set(f, a.val, GMP_RNDN);
-		mpfr_clear(a.val);
+		num_set(f, a.val);
+		num_free(a.val);
 	    }
 	    // get the number
 	    {
 		char junk;
 
-		varvalue = num_to_str_complex(f, 10, 0, -1, 1, &junk);
+		// This value must fully reproduce the contents of f (thus, the -2 in arg 4)
+		varvalue = num_to_str_complex(f, 10, 0, -2, 1, &junk);
 	    }
-	    mpfr_clear(f);
+	    num_free(f);
 	} else {		       // not a known var: itza literal (e.g. cos)
 	    varvalue = (char *)strdup(varname);
 	}
@@ -349,7 +355,7 @@ static int recursion(char *str)
     return retval;
 }				       /*}}} */
 
-int find_recursion(char *instring)
+static int find_recursion(char *instring)
 {				       /*{{{ */
     List vl = NULL;
     int retval;
@@ -360,7 +366,7 @@ int find_recursion(char *instring)
     return retval;
 }				       /*}}} */
 
-int find_recursion_core(List oldvars)
+static int find_recursion_core(List oldvars)
 {				       /*{{{ */
     List newvars = NULL;
     ListIterator oldvarsIterator;
@@ -372,7 +378,7 @@ int find_recursion_core(List oldvars)
     if (a.err)
 	return 0;
     if (!a.exp) {
-	mpfr_clear(a.val);
+	num_free(a.val);
 	return 0;
     }
 
@@ -385,19 +391,14 @@ int find_recursion_core(List oldvars)
 	while ((oldVarname =
 		(char *)nextListElement(oldvarsIterator)) != NULL) {
 	    if (!strcmp(newVarname, oldVarname)) {
-		unsigned int len = strlen(newVarname) + 73;
-		char *error = malloc(sizeof(char) * len);
-
-		snprintf(error, len,
-			 "%s was found twice in symbol descent. Recursive variables are not allowed.",
-			 newVarname);
-		report_error(error);
+		report_error
+		    ("%s was found twice in symbol descent. Recursive variables are not allowed.",
+		     newVarname);
 		// free the rest of the newvars list
 		do {
 		    free(newVarname);
 		} while ((newVarname =
 			  (char *)getHeadOfList(newvars)) != NULL);
-		free(error);
 		freeListIterator(oldvarsIterator);
 		return 1;
 	    }
@@ -420,30 +421,90 @@ int find_recursion_core(List oldvars)
     return retval;
 }				       /*}}} */
 
-void report_error(char *err)
+void report_error(const char *err_fmt, ...)
 {				       /*{{{ */
     extern char *errstring;
+    extern int errloc;
+    extern int column;
+    extern int lines;
+    extern int show_line_numbers;
     char *tempstring;
     unsigned int len;
+    va_list ap;
+    char *this_error;
+
+    va_start(ap, err_fmt);
+
+    this_error = calloc(strlen(err_fmt) + 1000, sizeof(char));
+    vsnprintf(this_error, strlen(err_fmt) + 1000, err_fmt, ap);
+    len = strlen(this_error) + 100;
+
+    va_end(ap);
+
+    /* okay, now this_error has the current error text in it */
 
     if (errstring) {
-	len = strlen(errstring) + 2 + strlen(err);
-	tempstring = calloc(sizeof(char), len);
-	snprintf(tempstring, len, "%s\n%s", errstring, err);
+	len += strlen(errstring);
+    }
+    tempstring = calloc(len, sizeof(char));
+    if (errstring) {
+	if (show_line_numbers) {
+	    snprintf(tempstring, len, "%s\nError on line %i: %s", errstring,
+		     lines, this_error);
+	} else {
+	    snprintf(tempstring, len, "%s\n%s", errstring, this_error);
+	}
 	free(errstring);
-	errstring = tempstring;
     } else {
-	errstring = (char *)strdup(err);
+	if (show_line_numbers) {
+	    snprintf(tempstring, len, "Error on line %i: %s", lines,
+		     this_error);
+	} else {
+	    snprintf(tempstring, len, "%s", this_error);
+	}
+    }
+    errstring = tempstring;
+    free(this_error);
+    if (errloc == -1) {
+	errloc = column;
     }
 }				       /*}}} */
 
-void set_prettyanswer(mpfr_t num)
+void display_and_clear_errstring()
+{				       /*{{{ */
+    extern int scanerror;
+    extern char *errstring;
+    extern int errloc;
+
+    if (errstring && errstring[0]) {
+	if (errloc != -1) {
+	    int i;
+
+	    fprintf(stderr, "   ");
+	    for (i = 0; i < errloc; i++) {
+		fprintf(stderr, " ");
+	    }
+	    fprintf(stderr, "^\n");
+	    errloc = -1;
+	}
+	fprintf(stderr, "%s", errstring);
+	if (errstring[strlen(errstring) - 1] != '\n')
+	    fprintf(stderr, "\n");
+	free(errstring);
+	errstring = NULL;
+	scanerror = 0;
+    }
+}				       /*}}} */
+
+void set_prettyanswer(const Number num)
 {				       /*{{{ */
     char *temp;
 
+    Dprintf("set_prettyanswer\n");
     if (pretty_answer) {
 	free(pretty_answer);
     }
+    Dprintf("set_prettyanswer - call print_this_result\n");
     temp = print_this_result(num);
     Dprintf("set_prettyanswer: %s\n", temp);
     if (temp) {
@@ -451,12 +512,13 @@ void set_prettyanswer(mpfr_t num)
     } else {
 	pretty_answer = NULL;
     }
+    Dprintf("set_prettyanswer - done\n");
 }				       /*}}} */
 
-char *print_this_result_dbl(double result)
+static char *print_this_result_dbl(const double result)
 {				       /*{{{ */
-    static char format[10];
-    static char *pa = NULL, *tmp;
+    char format[10];
+    static char *tmp;
     static char pa_dyn = 1;
     extern char *errstring;
     unsigned int decimal_places = 0;
@@ -478,14 +540,33 @@ char *print_this_result_dbl(double result)
 		return pa;
 	    } else
 		pa = tmp;
-	    if (conf.precision > -1 && !conf.engineering) {
-		snprintf(format, 10, "%%1.%if", conf.precision);
-		decimal_places = conf.precision;
-	    } else if (conf.precision > -1 && conf.engineering) {
-		snprintf(format, 10, "%%1.%iE", conf.precision);
+	    if (conf.precision > -1) {
 		decimal_places = conf.precision;
+		switch(conf.engineering) {
+		    case never:
+			snprintf(format, 10, "%%1.%if", conf.precision);
+			break;
+		    case always:
+			snprintf(format, 10, "%%1.%ie", conf.precision);
+			break;
+		    case automatic:
+			snprintf(format, 10, "%%1.%ig", conf.precision);
+			break;
+		}
+		Dprintf("precision was specified as %i, format string is \"%s\"\n", conf.precision, format);
 	    } else {
-		snprintf(format, 10, "%%G");
+		switch (conf.engineering) {
+		    case never:
+			strncpy(format, "%f", 10);
+			break;
+		    case always:
+			strncpy(format, "%e", 10);
+			break;
+		    case automatic:
+			strncpy(format, "%g", 10);
+			break;
+		}
+		Dprintf("precision is automatic, format string is \"%s\"\n", format);
 		if (fabs(result) < 10.0) {
 		    decimal_places = 6;
 		} else if (fabs(result) < 100.0) {
@@ -582,30 +663,30 @@ char *print_this_result_dbl(double result)
     } else {
 	char *curs;
 
-	Dprintf("normal numbers\n");
+	Dprintf("normal numbers (format: %s)\n", format);
 	switch (conf.output_format) {  /*{{{ */
 	    case DECIMAL_FORMAT:
 	    {
 		double junk;
 
+		Dprintf("fabs = %f, conf.engineering = %i, conf.print_ints = %i\n", fabs(modf(result, &junk)), conf.engineering, conf.print_ints);
 		/* This is the big call */
-		if (fabs(modf(result, &junk)) != 0.0 || conf.engineering ||
+		/* translation: if we don't have to handle the print_ints special case,
+		 * then we can just use the existing format. */
+		if (fabs(modf(result, &junk)) != 0.0 ||
 		    !conf.print_ints) {
 		    snprintf(pa, 310, format, result);
 		} else {
+		    /* this is the print_ints special case
+		     * (note that we strip trailing zeros) */
 		    snprintf(pa, 310, "%1.0f", result);
 		}
+		Dprintf("pa (unlocalized): %s\n", pa);
 		/* was it as good for you as it was for me?
 		 * now, you must localize it */
-		{
-		    unsigned int index;
-
-		    for (index = 0; index < strlen(pa); ++index) {
-			if (pa[index] == '.')
-			    pa[index] = conf.dec_delimiter;
-		    }
-		}
-		Dprintf("pa: %s\n",pa);
+		strswap('.',conf.dec_delimiter,pa);
+		
+		Dprintf("pa: %s\n", pa);
 		switch (conf.rounding_indication) {
 		    case SIMPLE_ROUNDING_INDICATION:
 			Dprintf("simple\n");
@@ -618,7 +699,7 @@ char *print_this_result_dbl(double result)
 			if (sig_figs < UINT32_MAX) {
 			    unsigned int t = count_digits(pa);
 
-			    Dprintf("digits in pa: %u (%u)\n",t,sig_figs);
+			    Dprintf("digits in pa: %u (%u)\n", t, sig_figs);
 			    if (pa[0] == '0' && pa[1] != '\0') {
 				--t;
 			    } else if (pa[0] == '-' && pa[1] == '0') {
@@ -684,16 +765,19 @@ char *print_this_result_dbl(double result)
 		    pa[1] = 'b';
 		}
 		// print it
-		for (i = conf.print_prefixes * 2; place >= 0; ++i) {
-		    double t = pow(2.0, place);
+		{
+		    double temp = result;
+		    for (i = conf.print_prefixes * 2; place >= 0; ++i) {
+			double t = pow(2.0, place);
 
-		    if (result >= t) {
-			pa[i] = '1';
-			result -= t;
-		    } else {
-			pa[i] = '0';
+			if (temp >= t) {
+			    pa[i] = '1';
+			    temp -= t;
+			} else {
+			    pa[i] = '0';
+			}
+			--place;
 		    }
-		    --place;
 		}
 		pa[i + 1] = 0;
 
@@ -724,12 +808,7 @@ char *print_this_result_dbl(double result)
 
     if (standard_output) {
 	if (errstring && strlen(errstring)) {
-	    extern int scanerror;
-
-	    fprintf(stderr, "%s\n", errstring);
-	    free(errstring);
-	    errstring = NULL;
-	    scanerror = 0;
+	    display_and_clear_errstring();
 	}
 	printf("%s%s\n",
 	       conf.print_equal ? (not_all_displayed ? "~= " : " = ")
@@ -739,50 +818,13 @@ char *print_this_result_dbl(double result)
     return pa;
 }				       /*}}} */
 
-int is_mpfr_int(mpfr_t potential_int)
+char *print_this_result(const Number result)
 {				       /*{{{ */
-    char *str, *curs;
-    mp_exp_t eptr;
-    int base;
-
-    switch (conf.output_format) {
-	case HEXADECIMAL_FORMAT:
-	    base = 16;
-	    break;
-	default:
-	case DECIMAL_FORMAT:
-	    base = 10;
-	    break;
-	case OCTAL_FORMAT:
-	    base = 8;
-	    break;
-	case BINARY_FORMAT:
-	    base = 2;
-	    break;
-    }
-    str = mpfr_get_str(NULL, &eptr, base, 0, potential_int, GMP_RNDN);
-    if (eptr < 0)
-	goto not_an_int;
-    curs = str + eptr;
-    while (curs && *curs) {
-	if (*curs != '0')
-	    goto not_an_int;
-	curs++;
-    }
-    mpfr_free_str(str);
-    return 1;
-  not_an_int:
-    mpfr_free_str(str);
-    return 0;
-}				       /*}}} */
-
-char *print_this_result(mpfr_t result)
-{				       /*{{{ */
-    static char *pa = NULL;
     extern char *errstring;
     unsigned int base = 0;
 
-    Dprintf("print_this_result (%f)\n", mpfr_get_d(result, GMP_RNDN));
+    Dprintf("print_this_result (%f) in format %i\n",
+	    num_get_d(result), conf.output_format);
     // output in the proper base and format
     switch (conf.output_format) {
 	case HEXADECIMAL_FORMAT:
@@ -799,12 +841,12 @@ char *print_this_result(mpfr_t result)
 	    // numbers like 5.1 that I don't even wanna begin to think about
 	    if (conf.precision_guard && conf.precision < 0) {
 		Dprintf("precision guard and automatic precision\n");
-		if (!conf.print_ints || !is_mpfr_int(result)) {
+		if (!conf.print_ints || !is_int(result)) {
 		    Dprintf("no print_ints or it isn't an int\n");
 		    //XXX: what is the following if() for?
 		    //if (mpfr_get_d(result, GMP_RNDN) !=
 		    //mpfr_get_si(result, GMP_RNDN)) {
-		    double res = mpfr_get_d(result, GMP_RNDN);
+		    double res = num_get_d(result);
 
 		    if (fabs(res) < DBL_EPSILON) {
 			res = 0.0;
@@ -828,10 +870,10 @@ char *print_this_result(mpfr_t result)
     not_all_displayed = 0;
     pa = num_to_str_complex(result, base, conf.engineering, conf.precision,
 			    conf.print_prefixes, &not_all_displayed);
-    Dprintf("not_all_displayed = %i\n",not_all_displayed);
+    Dprintf("not_all_displayed = %i\n", not_all_displayed);
 
     /* now, decide whether it's been rounded or not */
-    if (mpfr_inf_p(result) || mpfr_nan_p(result)) {
+    if (num_is_inf(result) || num_is_nan(result)) {
 	// if it is infinity, it's all there ;)
 	not_all_displayed = 0;
     } else if (not_all_displayed == 0) {
@@ -840,10 +882,11 @@ char *print_this_result(mpfr_t result)
 	    case SIMPLE_ROUNDING_INDICATION:
 	    {
 		char *pa2, junk;
+
 		Dprintf("simple full\n");
 
 		pa2 =
-		    num_to_str_complex(result, base, conf.engineering, -1,
+		    num_to_str_complex(result, base, conf.engineering, -2,
 				       conf.print_prefixes, &junk);
 		not_all_displayed = (strlen(pa) < strlen(pa2));
 		free(pa2);
@@ -855,7 +898,7 @@ char *print_this_result(mpfr_t result)
 		if (sig_figs < UINT32_MAX) {
 		    unsigned int t = count_digits(pa);
 
-		    Dprintf("digits in pa: %u (%u)\n",t,sig_figs);
+		    Dprintf("digits in pa: %u (%u)\n", t, sig_figs);
 		    not_all_displayed = (t < sig_figs);
 		} else {
 		    not_all_displayed = 0;
@@ -881,14 +924,9 @@ char *print_this_result(mpfr_t result)
 	}
     }
 
-  if (standard_output) {
+    if (standard_output) {
 	if (errstring && strlen(errstring)) {
-	    extern int scanerror;
-
-	    fprintf(stderr, "%s\n", errstring);
-	    free(errstring);
-	    errstring = NULL;
-	    scanerror = 0;
+	    display_and_clear_errstring();
 	}
 	printf("%s%s\n",
 	       conf.print_equal ? (not_all_displayed ? "~= " : " = ")
@@ -898,155 +936,118 @@ char *print_this_result(mpfr_t result)
     return pa;
 }				       /*}}} */
 
-void simple_exp(mpfr_t output, mpfr_t first, enum operations op,
-		mpfr_t second)
+void simple_exp(Number output, const Number first, const enum operations op,
+		const Number second)
 {				       /*{{{ */
     if (compute) {
-	mpfr_t temp;
+	Number temp;
 
-	mpfr_init(temp);
-	Dprintf("simple_exp: %f %i %f\n", mpfr_get_d(first, GMP_RNDN), op,
-		mpfr_get_d(second, GMP_RNDN));
+	num_init(temp);
+	Dprintf("simple_exp: %f %i %f\n", num_get_d(first), op,
+		num_get_d(second));
 
 	switch (op) {
 	    default:
-		mpfr_set_d(output, 0.0, GMP_RNDN);
+		num_set_d(output, 0.0);
 		break;
 	    case wequal:
-		mpfr_set_ui(output, mpfr_equal_p(first, second), GMP_RNDN);
+		num_set_ui(output, num_is_equal(first, second));
 		break;
 	    case wnequal:
-		mpfr_set_ui(output, !mpfr_equal_p(first, second), GMP_RNDN);
+		num_set_ui(output, !num_is_equal(first, second));
 		break;
 	    case wgt:
-		mpfr_set_ui(output, mpfr_greater_p(first, second), GMP_RNDN);
+		num_set_ui(output, num_is_greater(first, second));
 		break;
 	    case wlt:
-		mpfr_set_ui(output, mpfr_less_p(first, second), GMP_RNDN);
+		num_set_ui(output, num_is_less(first, second));
 		break;
 	    case wgeq:
-		mpfr_set_ui(output, mpfr_greaterequal_p(first, second),
-			    GMP_RNDN);
+		num_set_ui(output, num_is_greaterequal(first, second));
 		break;
 	    case wleq:
-		mpfr_set_ui(output, mpfr_lessequal_p(first, second),
-			    GMP_RNDN);
+		num_set_ui(output, num_is_lessequal(first, second));
 		break;
 	    case wplus:
-		mpfr_add(output, first, second, GMP_RNDN);
+		num_add(output, first, second);
 		break;
 	    case wminus:
-		mpfr_sub(output, first, second, GMP_RNDN);
+		num_sub(output, first, second);
 		break;
 	    case wmult:
-		mpfr_mul(output, first, second, GMP_RNDN);
+		num_mul(output, first, second);
 		break;
 	    case wdiv:
-		mpfr_div(output, first, second, GMP_RNDN);
+		num_div(output, first, second);
 		break;
 	    case wpow:
-		mpfr_pow(output, first, second, GMP_RNDN);
+		num_pow(output, first, second);
 		break;
 	    case wor:
-		mpfr_set_ui(output, (!mpfr_zero_p(first)) ||
-			    (!mpfr_zero_p(second)), GMP_RNDN);
+		num_set_ui(output, (!num_is_zero(first)) ||
+			    (!num_is_zero(second)));
 		break;
 	    case wand:
-		mpfr_set_ui(output, (!mpfr_zero_p(first)) &&
-			    (!mpfr_zero_p(second)), GMP_RNDN);
+		num_set_ui(output, (!num_is_zero(first)) &&
+			    (!num_is_zero(second)));
 		break;
 	    case wbor:
-	    {
-		mpz_t intfirst, intsecond, intoutput;
-
-		mpz_init(intfirst);
-		mpz_init(intsecond);
-		mpz_init(intoutput);
-		mpfr_get_z(intfirst, first, GMP_RNDN);
-		mpfr_get_z(intsecond, second, GMP_RNDN);
-		mpz_ior(intoutput, intfirst, intsecond);
-		mpfr_set_z(output, intoutput, GMP_RNDN);
-		mpz_clear(intfirst);
-		mpz_clear(intsecond);
-		mpz_clear(intoutput);
-	    }
+		num_bor(output, first, second);
 		break;
 	    case wband:
-	    {
-		mpz_t intfirst, intsecond, intoutput;
-
-		mpz_init(intfirst);
-		mpz_init(intsecond);
-		mpz_init(intoutput);
-		mpfr_get_z(intfirst, first, GMP_RNDN);
-		mpfr_get_z(intsecond, second, GMP_RNDN);
-		mpz_and(intoutput, intfirst, intsecond);
-		mpfr_set_z(output, intoutput, GMP_RNDN);
-		mpz_clear(intfirst);
-		mpz_clear(intsecond);
-		mpz_clear(intoutput);
-	    }
+		num_band(output, first, second);
 		break;
 	    case wbxor:
-	    {
-		mpz_t intfirst, intsecond, intoutput;
-
-		mpz_init(intfirst);
-		mpz_init(intsecond);
-		mpz_init(intoutput);
-		mpfr_get_z(intfirst, first, GMP_RNDN);
-		mpfr_get_z(intsecond, second, GMP_RNDN);
-		mpz_xor(intoutput, intfirst, intsecond);
-		mpfr_set_z(output, intoutput, GMP_RNDN);
-		mpz_clear(intfirst);
-		mpz_clear(intsecond);
-		mpz_clear(intoutput);
-	    }
+		num_bxor(output, first, second);
 		break;
 	    case wlshft:
-		mpfr_set_ui(temp, 2, GMP_RNDN);
-		mpfr_pow(temp, temp, second, GMP_RNDN);
-		mpfr_mul(output, first, temp, GMP_RNDN);
+		num_set_ui(temp, 2);
+		num_pow(temp, temp, second);
+		num_mul(output, first, temp);
 		break;
 	    case wrshft:
-		mpfr_set_ui(temp, 2, GMP_RNDN);
-		mpfr_pow(temp, temp, second, GMP_RNDN);
-		mpfr_div(output, first, temp, GMP_RNDN);
+		num_set_ui(temp, 2);
+		num_pow(temp, temp, second);
+		num_div(output, first, temp);
 		break;
 	    case wmod:
-		if (mpfr_zero_p(second)) {
-		    mpfr_set_nan(output);
+		if (num_is_zero(second)) {
+		    num_set_nan(output);
 		} else {
-		    /* find the biggest integer (temp) for which
-		     * first-temp*second is positive, then return the value
-		     * first-temp*second */
-		    mpfr_set_ui(temp, 0, GMP_RNDN);
-		    while (mpfr_cmp_ui(output, 0) > 0) {
-			mpfr_add_ui(temp, temp, 1, GMP_RNDN);	// temp++
-			mpfr_mul(output, temp, second, GMP_RNDN);	// temp*second
-			mpfr_sub(output, first, output, GMP_RNDN);
+		    /* divide, round to zero, multiply, subtract
+		     *
+		     * in essence, find the value x in the equation:
+		     * first = second * temp + x */
+		    num_div(output, first, second);
+		    if (conf.c_style_mod) {
+			num_rintz(output, output);	// makes zeros work
+		    } else {
+			if (num_sign(first) >= 0) {
+			    num_floor(output, output);
+			} else {
+			    num_ceil(output, output);
+			}
 		    }
-		    mpfr_sub_ui(temp, temp, 1, GMP_RNDN);
-		    mpfr_mul(output, temp, second, GMP_RNDN);
-		    mpfr_sub(output, first, output, GMP_RNDN);
+		    num_mul(output, output, second);
+		    num_sub(output, first, output);
 		}
 		break;
 	}
-	Dprintf("returns: %f\n", mpfr_get_d(output, GMP_RNDN));
-	mpfr_clear(temp);
+	Dprintf("returns: %f\n", num_get_d(output));
+	num_free(temp);
 	return;
     } else {
-	mpfr_set_ui(output, 0, GMP_RNDN);
+	num_set_ui(output, 0);
 	return;
     }
 }				       /*}}} */
 
-void uber_function(mpfr_t output, enum functions func, mpfr_t input)
+void uber_function(Number output, const enum functions func, Number input)
 {				       /*{{{ */
     if (compute) {
-	mpfr_t temp;
+	Number temp;
 
-	mpfr_init(temp);
+	num_init(temp);
 	if (!conf.use_radians) {
 	    switch (func) {
 		case wsin:
@@ -1055,9 +1056,9 @@ void uber_function(mpfr_t output, enum functions func, mpfr_t input)
 		case wcot:
 		case wsec:
 		case wcsc:
-		    mpfr_const_pi(temp, GMP_RNDN);
-		    mpfr_mul(input, input, temp, GMP_RNDN);
-		    mpfr_div_ui(input, input, 180, GMP_RNDN);
+		    num_const_pi(temp);
+		    num_mul(input, input, temp);
+		    num_div_ui(input, input, 180);
 		    break;
 		case wasin:
 		case wacos:
@@ -1065,9 +1066,9 @@ void uber_function(mpfr_t output, enum functions func, mpfr_t input)
 		case wacot:
 		case wasec:
 		case wacsc:
-		    mpfr_const_pi(temp, GMP_RNDN);
-		    mpfr_pow_si(temp, temp, -1, GMP_RNDN);
-		    mpfr_mul_ui(temp, temp, 180, GMP_RNDN);
+		    num_const_pi(temp);
+		    num_pow_si(temp, temp, -1);
+		    num_mul_ui(temp, temp, 180);
 		    break;
 		default:
 		    break;
@@ -1075,274 +1076,201 @@ void uber_function(mpfr_t output, enum functions func, mpfr_t input)
 	}
 	switch (func) {
 	    case wsin:
-		mpfr_sin(output, input, GMP_RNDN);
+		num_sin(output, input);
 		break;
 	    case wcos:
-		mpfr_cos(output, input, GMP_RNDN);
+		num_cos(output, input);
 		break;
 	    case wtan:
-		mpfr_tan(output, input, GMP_RNDN);
+		num_tan(output, input);
 		break;
 	    case wcot:
-#ifdef HAVE_MPFR_22
-		mpfr_cot(output, input, GMP_RNDN);
-#else
-		mpfr_tan(output, input, GMP_RNDN);
-		mpfr_pow_si(output, output, -1, GMP_RNDN);
-#endif
+		num_cot(output, input);
 		break;
 	    case wsec:
-#ifdef HAVE_MPFR_22
-		mpfr_sec(output, input, GMP_RNDN);
-#else
-		mpfr_cos(output, input, GMP_RNDN);
-		mpfr_pow_si(output, output, -1, GMP_RNDN);
-#endif
+		num_sec(output, input);
 		break;
 	    case wcsc:
-#ifdef HAVE_MPFR_22
-		mpfr_csc(output, input, GMP_RNDN);
-#else
-		mpfr_sin(output, input, GMP_RNDN);
-		mpfr_pow_si(output, output, -1, GMP_RNDN);
-#endif
+		num_csc(output, input);
 		break;
 	    case wasin:
-		mpfr_asin(output, input, GMP_RNDN);
+		num_asin(output, input);
 		if (!conf.use_radians) {
-		    mpfr_mul(output, output, temp, GMP_RNDN);
+		    num_mul(output, output, temp);
 		}
 		break;
 	    case wacos:
-		mpfr_acos(output, input, GMP_RNDN);
+		num_acos(output, input);
 		if (!conf.use_radians) {
-		    mpfr_mul(output, output, temp, GMP_RNDN);
+		    num_mul(output, output, temp);
 		}
 		break;
 	    case watan:
-		mpfr_atan(output, input, GMP_RNDN);
+		num_atan(output, input);
 		if (!conf.use_radians) {
-		    mpfr_mul(output, output, temp, GMP_RNDN);
+		    num_mul(output, output, temp);
 		}
 		break;
 	    case wacot:
-		mpfr_pow_si(output, input, -1, GMP_RNDN);
-		mpfr_atan(output, output, GMP_RNDN);
+		num_pow_si(output, input, -1);
+		num_atan(output, output);
 		if (!conf.use_radians) {
-		    mpfr_mul(output, output, temp, GMP_RNDN);
+		    num_mul(output, output, temp);
 		}
 		break;
 	    case wasec:
-		mpfr_pow_si(output, input, -1, GMP_RNDN);
-		mpfr_acos(output, output, GMP_RNDN);
+		num_pow_si(output, input, -1);
+		num_acos(output, output);
 		if (!conf.use_radians) {
-		    mpfr_mul(output, output, temp, GMP_RNDN);
+		    num_mul(output, output, temp);
 		}
 		break;
 	    case wacsc:
-		mpfr_pow_si(output, input, -1, GMP_RNDN);
-		mpfr_asin(output, output, GMP_RNDN);
+		num_pow_si(output, input, -1);
+		num_asin(output, output);
 		if (!conf.use_radians) {
-		    mpfr_mul(output, output, temp, GMP_RNDN);
+		    num_mul(output, output, temp);
 		}
 		break;
 	    case wsinh:
-		mpfr_sinh(output, input, GMP_RNDN);
+		num_sinh(output, input);
 		break;
 	    case wcosh:
-		mpfr_cosh(output, input, GMP_RNDN);
+		num_cosh(output, input);
 		break;
 	    case wtanh:
-		mpfr_tanh(output, input, GMP_RNDN);
+		num_tanh(output, input);
 		break;
 	    case wcoth:
-#ifdef HAVE_MPFR_22
-		mpfr_coth(output, input, GMP_RNDN);
-#else
-		mpfr_tanh(output, input, GMP_RNDN);
-		mpfr_pow_si(output, output, -1, GMP_RNDN);
-#endif
+		num_coth(output, input);
 		break;
 	    case wsech:
-#ifdef HAVE_MPFR_22
-		mpfr_sech(output, input, GMP_RNDN);
-#else
-		mpfr_cosh(output, input, GMP_RNDN);
-		mpfr_pow_si(output, output, -1, GMP_RNDN);
-#endif
+		num_sech(output, input);
 		break;
 	    case wcsch:
-#ifdef HAVE_MPFR_22
-		mpfr_csch(output, input, GMP_RNDN);
-#else
-		mpfr_sinh(output, input, GMP_RNDN);
-		mpfr_pow_si(output, output, -1, GMP_RNDN);
-#endif
+		num_csch(output, input);
 		break;
 	    case wasinh:
-		mpfr_asinh(output, input, GMP_RNDN);
+		num_asinh(output, input);
 		break;
 	    case wacosh:
-		mpfr_acosh(output, input, GMP_RNDN);
+		num_acosh(output, input);
 		break;
 	    case watanh:
-		mpfr_atanh(output, input, GMP_RNDN);
+		num_atanh(output, input);
 		break;
 	    case wacoth:
-		mpfr_pow_si(input, input, -1, GMP_RNDN);
-		mpfr_atanh(output, input, GMP_RNDN);
+		num_acoth(output, input);
 		break;
 	    case wasech:
-		mpfr_pow_si(output, input, -1, GMP_RNDN);
-		mpfr_acosh(output, output, GMP_RNDN);
+		num_asech(output, input);
 		break;
 	    case wacsch:
-		mpfr_pow_si(output, input, -1, GMP_RNDN);
-		mpfr_asinh(output, output, GMP_RNDN);
+		num_acsch(output, input);
 		break;
 	    case wlog:
-		mpfr_log10(output, input, GMP_RNDN);
+		num_log10(output, input);
 		break;
 	    case wlogtwo:
-		mpfr_log2(output, input, GMP_RNDN);
+		num_log2(output, input);
 		break;
 	    case wln:
-		mpfr_log(output, input, GMP_RNDN);
+		num_log(output, input);
 		break;
 	    case wround:
-		mpfr_rint(output, input, GMP_RNDN);
+		num_rint(output, input);
 		break;
 	    case wneg:
-		mpfr_mul_si(output, input, -1, GMP_RNDN);
+		num_neg(output, input);
 		break;
 	    case wnot:
-		mpfr_set_ui(output, mpfr_zero_p(input), GMP_RNDN);
+		num_set_ui(output, num_is_zero(input));
 		break;
 	    case wabs:
-		mpfr_abs(output, input, GMP_RNDN);
+		num_abs(output, input);
 		break;
 	    case wsqrt:
-		mpfr_sqrt(output, input, GMP_RNDN);
+		num_sqrt(output, input);
 		break;
 	    case wfloor:
-		mpfr_floor(output, input);
+		num_floor(output, input);
 		break;
 	    case wceil:
-		mpfr_ceil(output, input);
+		num_ceil(output, input);
 		break;
 	    case wrand:
-		seed_random();
-		while (mpfr_urandomb(output, randstate) != 0) ;
-		mpfr_mul(output, output, input, GMP_RNDN);
-		if (mpfr_cmp_si(input, 0) < 0) {
-		    mpfr_mul_si(output, output, -1, GMP_RNDN);
+		while (num_random(output) != 0) ;
+		num_mul(output, output, input);
+		if (num_cmp_si(input, 0) < 0) {
+		    num_mul_si(output, output, -1);
 		}
 		break;
 	    case wirand:
-		seed_random();
-		while (mpfr_urandomb(output, randstate) != 0) ;
-		mpfr_mul(output, output, input, GMP_RNDN);
-		if (mpfr_cmp_si(input, 0) < 0) {
-		    mpfr_mul_si(output, output, -1, GMP_RNDN);
+		while (num_random(output) != 0) ;
+		num_mul(output, output, input);
+		if (num_cmp_si(input, 0) < 0) {
+		    num_mul_si(output, output, -1);
 		}
-		mpfr_rint(output, output, GMP_RNDN);
+		num_rint(output, output);
 		break;
 	    case wcbrt:
-		mpfr_cbrt(output, input, GMP_RNDN);
+		num_cbrt(output, input);
 		break;
 	    case wexp:
-		mpfr_exp(output, input, GMP_RNDN);
+		num_exp(output, input);
 		break;
 	    case wfact:
-		mpfr_fac_ui(output, mpfr_get_ui(input, GMP_RNDN), GMP_RNDN);
+		num_factorial(output, num_get_ui(input));
 		break;
 	    case wcomp:
-	    {
-		mpz_t integer, intout;
-
-		mpz_init(integer);
-		mpz_init(intout);
-		mpfr_get_z(integer, input, GMP_RNDN);
-		mpz_com(intout, integer);
-		mpfr_set_z(output, intout, GMP_RNDN);
-		mpz_clear(integer);
-		mpz_clear(intout);
-	    }
+		num_comp(output, input);
 		break;
 #ifdef HAVE_MPFR_22
 	    case weint:
-		mpfr_eint(output, input, GMP_RNDN);
+		num_eint(output, input);
 		break;
 #endif
 	    case wgamma:
-		mpfr_gamma(output, input, GMP_RNDN);
+		num_gamma(output, input);
 		break;
 #ifdef HAVE_MPFR_22
 	    case wlngamma:
-		mpfr_lngamma(output, input, GMP_RNDN);
+		num_lngamma(output, input);
 		break;
 #endif
 	    case wzeta:
-		mpfr_zeta(output, input, GMP_RNDN);
+		num_zeta(output, input);
 		break;
 	    case wsinc:
-		if (mpfr_zero_p(input)) {
-		    mpfr_set_ui(output, 1, GMP_RNDN);
-		} else {
-		    mpfr_sin(output, input, GMP_RNDN);
-		    mpfr_div(output, output, input, GMP_RNDN);
-		}
+		num_sinc(output, input);
 		break;
 	    case wbnot:
-#ifdef _MPFR_H_HAVE_INTMAX_T
-		mpfr_set_uj(output, ~mpfr_get_uj(input, GMP_RNDN), GMP_RNDN);
-#else
-		mpfr_set_ui(output, ~mpfr_get_ui(input, GMP_RNDN), GMP_RNDN);
-#endif
+		num_bnot(output, input);
 		break;
 	    default:
-		mpfr_set(output, input, GMP_RNDN);
+		num_set(output, input);
 		break;
 	}
-	mpfr_clear(temp);
+	num_free(temp);
 	return;
     } else {
-	mpfr_set_ui(output, 0, GMP_RNDN);
+	num_set_ui(output, 0);
 	return;
     }
 }				       /*}}} */
 
-int seed_random(void)
-{				       /*{{{ */
-    static char seeded = 0;
-
-    if (!seeded) {
-	struct timeval tp;
-
-	if (gettimeofday(&tp, NULL) != 0) {
-	    perror("gettimeofday");
-	    exit(EXIT_FAILURE);
-	}
-	gmp_randinit_default(randstate);
-	gmp_randseed_ui(randstate, (unsigned long)(tp.tv_usec));
-	//srandom(time(NULL));
-	seeded = 1;
-    }
-    return 1;
-}				       /*}}} */
-
-char *output_string(unsigned int o)
+char *output_string(const unsigned int o)
 {				       /*{{{ */
     switch (o) {
 	case HEXADECIMAL_FORMAT:
 	    return "hexadecimal format (0xf)";
 	case OCTAL_FORMAT:
-	    return "octal format (08)";
+	    return "octal format (08)       ";
 	case BINARY_FORMAT:
-	    return "binary format (0b1)";
+	    return "binary format (0b1)     ";
 	case DECIMAL_FORMAT:
-	    return "decimal format (9)";
+	    return "decimal format (9)      ";
 	default:
-	    return "error, unknown format";
+	    return "error, unknown format   ";
     }
 }				       /*}}} */
