@@ -15,8 +15,7 @@
 #endif
 #include "definitions.h"
 
-#include <gmp.h>
-#include <mpfr.h>
+#include "number.h"
 
 #ifdef EBUG
 #include <stdio.h>
@@ -100,28 +99,32 @@ enum operations
 enum commands
 { redisplay, nothing };
 
-void parseme(char *);
-void report_error(char *);
-void set_prettyanswer(mpfr_t num);
-char *print_this_result(mpfr_t result);
-void uber_function(mpfr_t output, enum functions func, mpfr_t input);
-void simple_exp(mpfr_t output, mpfr_t first, enum operations op,
-				mpfr_t second);
+enum engineering_modes
+{ always, never, automatic };
+
+void parseme(const char *);
+void report_error(const char * fmt, ...);
+void display_and_clear_errstring(void);
+void set_prettyanswer(const Number num);
+char *print_this_result(const Number result);
+void uber_function(Number output, const enum functions func, Number input);
+void simple_exp(Number output, const Number first, const enum operations op,
+				const Number second);
 int seed_random(void);
-char *output_string(unsigned int);
+char *output_string(const unsigned int);
 
 struct _conf
 {
     int precision;
-    unsigned int engineering:1;
+    enum engineering_modes engineering:2;
     unsigned int picky_variables:1;
     unsigned int use_radians:1;
     unsigned int output_format:4;
     unsigned int print_prefixes:1;
     unsigned int rounding_indication:4;
     unsigned int remember_errors:1;
-    unsigned char thou_delimiter;
-    unsigned char dec_delimiter;
+    char thou_delimiter;
+    char dec_delimiter;
     unsigned int precision_guard:1;
     unsigned int history_limit:1;
     unsigned long history_limit_len;
@@ -130,13 +133,15 @@ struct _conf
     unsigned int simple_calc:1;
     unsigned int verbose:1;
     unsigned int print_commas:1;
+    unsigned int live_precision:1;
+    unsigned int c_style_mod:1;
 };
 
 /* configuration */
 extern struct _conf conf;
 
 /* results */
-extern mpfr_t last_answer;
+extern Number last_answer;
 extern char *pretty_answer;
 
 /* communication with parser */
@@ -147,9 +152,6 @@ extern unsigned int sig_figs;
 extern char standard_output;
 extern char not_all_displayed;
 
-/* random state */
-extern gmp_randstate_t randstate;
-
 #define DECIMAL_FORMAT 0
 #define OCTAL_FORMAT 1
 #define HEXADECIMAL_FORMAT 2
