@@ -2,8 +2,8 @@
 #include "config.h"
 #endif
 #include <string.h>
-#include <gmp.h>
-#include <mpfr.h>
+#include <ctype.h> /* for isspace() */
+#include "number.h"
 #include "uint32_max.h"
 #include "variables.h"
 #include "calculator.h"
@@ -13,34 +13,49 @@
 #include "isconst.h"
 #include "isfunc.h"
 #include "explain.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
-void explain_command(char *);
-void explain_variable(char *);
-void explain_constant(char *);
-void explain_function(char *);
+static void explain_command(const char *);
+static void explain_variable(const char *);
+static void explain_constant(const char *);
+static void explain_function(const char *);
 
-void explain(char *str)
+void explain(const char *str)
 {				       /*{{{ */
-    if (!str || !*str) {
+    size_t curs;
+    char * mystr;
+
+    if (str == NULL || str[0] == 0) {
+	printf("Nothing to explain.\n");
 	return;
     }
-    if (!strcmp(str, "q")) {
+    mystr = strdup(str);
+    /* for sanity's sake, remove any trailing whitespace */
+    curs = strlen(mystr) - 1;
+    while (isspace(mystr[curs]) && curs > 0) {
+	mystr[curs] = 0;
+	curs--;
+    }
+    if (!strcmp(mystr, "q")) {
 	printf("Quits the program.");
-    } else if (*str == '\\') {	       // it's a command
-	explain_command(str);
-    } else if (isconst(str)) {	       // it's a constant
-	explain_constant(str);
-    } else if (isfunc(str)) {	       // it's a function
-	explain_function(str);
-    } else if (varexists(str)) {       // it's a variable
-	explain_variable(str);
+    } else if (*mystr == '\\') {	       // it's a command
+	explain_command(mystr);
+    } else if (isconst(mystr)) {	       // it's a constant
+	explain_constant(mystr);
+    } else if (isfunc(mystr)) {	       // it's a function
+	explain_function(mystr);
+    } else if (varexists(mystr)) {       // it's a variable
+	explain_variable(mystr);
     } else {			       // it's a call for help
 	printf("%s is neither a command, constant, function, or variable.\n",
-	       str);
+	       mystr);
     }
+    free(mystr);
 }				       /*}}} */
 
-void explain_command(char *str)
+static void explain_command(const char *str)
 {				       /*{{{ */
     str++;
     if (!strcmp(str, "b") || !strcmp(str, "bin") || !strcmp(str, "binary")) {
@@ -104,7 +119,7 @@ void explain_command(char *str)
     } else if (!strcmp(str, "bits")) {
 	printf
 	    ("Sets the number of bits used internally to represent numbers. Used like this: \\bitsX where X is a number that must be above %li and below %li.\n",
-	     (long int)MPFR_PREC_MIN, (long int)MPFR_PREC_MAX);
+	     (long int)NUM_PREC_MIN, (long int)NUM_PREC_MAX);
     } else if (!strcmp(str, "save")) {
 	printf
 	    ("Saves the history and variable list to a file. Used like this: \\saveXXXXX where XXXXX is the name of the file to save.\n");
@@ -123,12 +138,15 @@ void explain_command(char *str)
     } else if (!strcmp(str, "explain")) {
 	printf
 	    ("Gives you information about commands, variables, constants and functions.\n");
+    } else if (!strcmp(str, "cmod")) {
+	printf
+	    ("Changes how the modulus operator (%%) behaves with negative numbers.\n The default is to behave like the C programming language modulus, the other is slightly more flexible. For example, with the default setting:\n\n\t-340 %% 60 == -40; 340 %% -60 == 40; -340 %% -60 == -40\n\nWhen this setting is toggled, it behaves like this:\n\n\t-340 %% 60 == -40; 340 %% -60 == -20; -340 %% -60 == 20\n");
     } else {
 	printf("Undefined command.\n");
     }
 }				       /*}}} */
 
-void explain_variable(char *str)
+static void explain_variable(const char *str)
 {				       /*{{{ */
     struct answer var;
 
@@ -144,20 +162,20 @@ void explain_variable(char *str)
 	printf("%s uses the following variables:\n", str);
 	strings = extract_vars(var.exp);
 	while (listLen(strings) > 0) {
-	    char *str = (char *)getHeadOfList(strings);
-	    char *value = evalvar(str);
+	    char *curstr = (char *)getHeadOfList(strings);
+	    char *value = evalvar(curstr);
 
-	    printf("\t%s\t(currently: %s)\n", str,
+	    printf("\t%s\t(currently: %s)\n", curstr,
 		   value ? value : "undefined");
-	    if (str) {
-		free(str);
+	    if (curstr) {
+		free(curstr);
 	    }
 	    if (value) {
 		free(value);
 	    }
 	}
+	freeList(&strings);
     } else {
-	extern char standard_output;
 	char std_save;
 
 	std_save = standard_output;
@@ -171,7 +189,7 @@ void explain_variable(char *str)
     }
 }				       /*}}} */
 
-void explain_constant(char *str)
+static void explain_constant(const char *str)
 {				       /*{{{ */
     if (!strcmp(str, "e")) {
 	printf
@@ -231,7 +249,7 @@ void explain_constant(char *str)
     } else if (!strcmp(str, "b")) {
 	printf
 	    ("Wein displacement constant. Defines the relationship between the thermodynamic temperature of a black body and the wavelength at which the intensity of the radiation is the greatest.");
-    } else if (!strcmp(str, "ao")) {
+    } else if (!strcmp(str, "a0")) {
 	printf
 	    ("Bohr radius. A single electron orbiting at the lowest possible energy in a hydrogen atom orbits at this distance.");
     } else if (!strcmp(str, "F")) {
@@ -293,14 +311,16 @@ void explain_constant(char *str)
 	printf
 	    ("Magnetic flux quantum. The quantum of magnetic flux passing through a superconductor.");
     } else if (!strcmp(str, "K")) {
-	printf("Catalan's constant commonly appears in estimates of combinatorial functions and in certain classes of sums and definite integrals.");
+	printf
+	    ("Catalan's constant commonly appears in estimates of combinatorial functions and in certain classes of sums and definite integrals.");
     } else if (!strcmp(str, "sinc")) {
-	printf("Sinus cardinalis, also known as the interpolation function, filtering function, or the first spherical Bessel function, is the product of a sine function and a monotonically decreasing function.");
+	printf
+	    ("Sinus cardinalis, also known as the interpolation function, filtering function, or the first spherical Bessel function, is the product of a sine function and a monotonically decreasing function.");
     }
     printf("\n");
 }				       /*}}} */
 
-void explain_function(char *str)
+static void explain_function(const char *str)
 {				       /*{{{ */
     if (!strcmp(str, "sin") || !strcmp(str, "cos") || !strcmp(str, "tan") ||
 	!strcmp(str, "cot")) {
@@ -331,8 +351,6 @@ void explain_function(char *str)
 	printf("The absolute value (the distance of the number from zero).");
     } else if (!strcmp(str, "sqrt")) {
 	printf("The square root function.");
-    } else if (!strcmp(str, "exp")) {
-	printf("Returns the exponential of the input number.");
     } else if (!strcmp(str, "floor")) {
 	printf
 	    ("Returns the biggest integer that is not bigger than the input number.");
@@ -350,11 +368,23 @@ void explain_function(char *str)
     } else if (!strcmp(str, "comp")) {
 	printf("Returns the one's complement of the input number.");
     } else if (!strcmp(str, "zeta")) {
-	printf("Returns the Riemann zeta function of the input number. This is used primarily in number theory, but also has applications in physics, probability theory, and applied statistics.");
+	printf
+	    ("Returns the Riemann zeta function of the input number. This is used primarily in number theory, but also has applications in physics, probability theory, and applied statistics.");
     } else if (!strcmp(str, "Gamma")) {
-	printf("Returns the Gamma function of the input number. The Gamma function extends the factorial function to complex and non-natural numbers.");
+	printf
+	    ("Returns the Gamma function of the input number. The Gamma function extends the factorial function to complex and non-natural numbers.");
     } else if (!strcmp(str, "lnGamma")) {
-	printf("Returns the natural log of the Gamma function of the input number.");
+	printf
+	    ("Returns the natural log of the Gamma function of the input number.");
+    } else if (!strcmp(str, "eint")) {
+	printf("The exponential integral function.");
+    } else if (!strcmp(str, "sinc")) {
+	printf("Returns the sampling function, which is 1 if the input is 0, or sin(x)/x otherwise.");
+    } else if (!strcmp(str, "exp")) {
+	printf("Returns the value of e to the given power. Equivalent to: e^");
+    } else {
+	printf
+	    ("Hrm... this function doesn't have any documentation! Pester the developer!");
     }
     printf("\n");
 }				       /*}}} */
